set echo on
clear screen
create directory hprof_dir as '/Users/robvanwijk/sql/'
/
insert into logger_prefs (pref_name,pref_value)
select 'SQL_TRACE_USER', 'OFF' from dual union all
select 'SQL_TRACE_MODULE', 'OFF' from dual union all
select 'HPROF_USER', 'OFF' from dual union all
select 'HPROF_MODULE', 'OFF' from dual union all
select 'HPROF_LOCATION', 'OFF' from dual union all
select 'HPROF_FILENAME', 'OFF' from dual
/
pause
create package mylogger
as
  procedure log_start (p_module in varchar2);
  procedure log (p_tekst in varchar2);
  procedure log_end (p_module in varchar2);
  procedure log_error (p_module in varchar2);
end mylogger;
/
pause
create package body mylogger
as
  procedure log_start(p_module in varchar2)
  is
  begin
    dbms_application_info.set_module(p_module,'Start')
    ;
    logger.log('Start',p_module)
    ;
    if    logger.get_pref('SQL_TRACE_USER') = user
      and logger.get_pref('SQL_TRACE_MODULE') = p_module
    then
      dbms_session.session_trace_enable
      ( waits     => true
      , binds     => true
      , plan_stat => 'ALL_EXECUTIONS'
      );
    end if
    ;
    if    logger.get_pref('HPROF_USER') = user
      and logger.get_pref('HPROF_MODULE') = p_module
      and logger.get_pref('HPROF_LOCATION') is not null
      and logger.get_pref('HPROF_FILENAME') is not null
    then
      dbms_hprof.start_profiling
      ( location => logger.get_pref('HPROF_LOCATION')
      , filename => logger.get_pref('HPROF_FILENAME')
      );
    end if;
  end log_start
  ;
  procedure log (p_tekst in varchar2)
  is
  begin
    logger.log(p_tekst);
  end log
  ;
  procedure log_end (p_module in varchar2)
  is
  begin
    dbms_application_info.set_module(p_module,'Einde')
    ;
    logger.log('Einde',p_module)
    ;
    if    logger.get_pref('SQL_TRACE_USER') = user
      and logger.get_pref('SQL_TRACE_MODULE') = p_module
    then
      dbms_session.session_trace_disable;
    end if
    ;
    if    logger.get_pref('HPROF_USER') = user
      and logger.get_pref('HPROF_MODULE') = p_module
    then
      dbms_hprof.stop_profiling;
    end if
    ;
  end log_end
  ;
  procedure log_error (p_module in varchar2)
  is
  begin
    logger.log_error();
    log_end(p_module);
  end log_error
  ;
end mylogger;
/
pause
clear screen
create package pkg
as
  procedure p;
end pkg;
/
pause
create package body pkg
as
  function maximum_salary return number
  is
    l_max_sal number; 
  begin
    select max(sal)
      into l_max_sal
      from emp
    ;
    mylogger.log('Maximum salary = ' || l_max_sal);
    return l_max_sal;
  end maximum_salary
  ;
  procedure p1
  is
  begin
    if 2 + 2 = 4
    then
      mylogger.log('2 + 2 = 4');
    else
      mylogger.log('2 + 2 != 4');
    end if;
  end p1
  ;
  procedure p2
  is
  begin
    for i in 1 .. 10
    loop
      if maximum_salary() = 42
      then
        mylogger.log('42');
      else
        mylogger.log('!= 42');
      end if;
    end loop;
  end p2
  ;
  procedure p3
  is
  begin
    mylogger.log('Before insert');
    insert into emp (empno,ename,sal) values (7777,'VAN WIJK',1000);
    mylogger.log('After insert, before delete');
    delete emp where empno = 7777;
    mylogger.log('After delete');
  end p3
  ;
  procedure p
  is
  begin
    mylogger.log_start('p');
    p1;
    p2;
    p3;
    mylogger.log_end('p');
  exception
  when others then
    mylogger.log_error('p');
    raise;
  end p
  ;
end pkg;
/
pause
exec logger.set_level('ERROR')
exec pkg.p
select * from logger_logs_5_min
/
pause
exec logger.set_level('DEBUG')
exec pkg.p
pause
select * from logger_logs_5_min
/
set echo off
delete logger_prefs
 where pref_name in
       ( 'SQL_TRACE_USER'
       , 'SQL_TRACE_MODULE'
       , 'HPROF_USER'
       , 'HPROF_MODULE'
       , 'HPROF_LOCATION'
       , 'HPROF_FILENAME'
       )
/
drop package pkg
/
drop package mylogger
/
drop directory hprof_dir
/
exec logger.purge_all
